__version__ = "0.2.9"
__git_commiter_name__ = "Guy Rozendorn"
__git_commiter_email__ = "guy@rzn.co.il"
__git_branch__ = '(detached from v0.2.9)'
__git_remote_tracking_branch__ = '(No remote tracking)'
__git_remote_url__ = '(Not remote tracking)'
__git_head_hash__ = 'bd0ce8f8e66f63d54beabab7fdb3b0ad20991f87'
__git_head_subject__ = 'Finished release v0.2.9.'
__git_head_message__ = '* release/v0.2.9:\n  TRIVIAL gitignore\n  buildout.cfg: changed isolated python version to v2.7.6.21\n  buildout.cfg: changed isolated python version to v2.7.6.20\n  buildout.cfg: changed isolated python version to v2.7.6.19'
__git_dirty_diff__ = ''